/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * <p>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.qlangtech.tis.order.center;

import com.qlangtech.tis.assemble.FullbuildPhase;
import com.qlangtech.tis.cloud.ITISCoordinator;
import com.qlangtech.tis.exec.AbstractActionInvocation;
import com.qlangtech.tis.exec.ActionInvocation;
import com.qlangtech.tis.exec.ExecutePhaseRange;
import com.qlangtech.tis.exec.ExecuteResult;
import com.qlangtech.tis.exec.impl.DefaultChainContext;
import com.qlangtech.tis.fullbuild.phasestatus.IFlush2Local;
import com.qlangtech.tis.fullbuild.phasestatus.IFlush2LocalFactory;
import com.qlangtech.tis.fullbuild.phasestatus.PhaseStatusCollection;
import com.qlangtech.tis.fullbuild.phasestatus.impl.BasicPhaseStatus;
import com.qlangtech.tis.fullbuild.phasestatus.impl.BuildPhaseStatus;
import com.qlangtech.tis.fullbuild.phasestatus.impl.DumpPhaseStatus;
import com.qlangtech.tis.fullbuild.phasestatus.impl.IndexBackFlowPhaseStatus;
import com.qlangtech.tis.fullbuild.phasestatus.impl.JoinPhaseStatus;
import com.qlangtech.tis.realtime.transfer.IOnsListenerStatus;
import com.qlangtech.tis.rpc.server.DefaultLoggerAppenderServiceImpl;
import com.qlangtech.tis.rpc.server.FullBuildStatCollectorServer;
import com.qlangtech.tis.rpc.server.IncrStatusServer;
import com.qlangtech.tis.rpc.server.IncrStatusUmbilicalProtocolImpl;
import com.qlangtech.tis.solrj.util.ZkUtils;
import org.apache.commons.daemon.Daemon;
import org.apache.commons.daemon.DaemonContext;
import org.apache.commons.daemon.DaemonInitException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

/**
 * @author 百岁（baisui@qlangtech.com）
 * @date 2015年11月5日 下午6:57:19
 */
public class IndexSwapTaskflowLauncher implements Daemon, ServletContextListener {

    private static final Logger logger = LoggerFactory.getLogger(IndexSwapTaskflowLauncher.class);

    public static final String KEY_INDEX_SWAP_TASK_FLOW_LAUNCHER = "IndexSwapTaskflowLauncher";
    private ITISCoordinator zkClient;

    static {
    }

    public static void initPhaseStatusStatusWriter() {

    }


    public static IndexSwapTaskflowLauncher getIndexSwapTaskflowLauncher(ServletContext context) {
        IndexSwapTaskflowLauncher result = (IndexSwapTaskflowLauncher) context.getAttribute(KEY_INDEX_SWAP_TASK_FLOW_LAUNCHER);
        if (result == null) {
            throw new IllegalStateException("IndexSwapTaskflowLauncher can not be null in servletContext");
        }
        return result;
    }

    @Override
    public void contextDestroyed(ServletContextEvent sce) {
    }


    public void setZkClient(ITISCoordinator zkClient) {
        this.zkClient = zkClient;
    }

    public ITISCoordinator getZkClient() {
        throw new UnsupportedOperationException();
    }


    private Collection<IOnsListenerStatus> incrChannels;

    public Collection<IOnsListenerStatus> getIncrChannels() {
        return incrChannels;
    }

    @Override
    public void contextInitialized(ServletContextEvent sce) {
        // AbstractTisCloudSolrClient.initHashcodeRouter();
        // 构建各阶段持久化
        try {
            this.afterPropertiesSet();
            this.incrChannels = initIncrTransferStateCollect();
            // FlumeApplication.startFlume();
            sce.getServletContext().setAttribute(KEY_INDEX_SWAP_TASK_FLOW_LAUNCHER, this);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public void afterPropertiesSet() throws Exception {
        this.setZkClient(ITISCoordinator.create());
    }

    private IncrStatusServer incrStatusServer;

    public IncrStatusServer getIncrStatusUmbilicalProtocol() {
        if (incrStatusServer == null) {
            throw new IllegalStateException("incrStatusUmbilicalProtocolServer can not be null");
        }
        return this.incrStatusServer;
    }

    // 发布增量集群任务收集器
    private Collection<IOnsListenerStatus> initIncrTransferStateCollect() throws Exception {
        // this.incrStatusUmbilicalProtocolServer = new IncrStatusUmbilicalProtocolImpl();
        final int exportPort = ZkUtils.ZK_ASSEMBLE_LOG_COLLECT_PORT; //NetUtils.getFreeSocketPort();
        incrStatusServer = new IncrStatusServer(exportPort);
        incrStatusServer.addService(IncrStatusUmbilicalProtocolImpl.getInstance());
        incrStatusServer.addService(DefaultLoggerAppenderServiceImpl.getInstance());
        incrStatusServer.addService(FullBuildStatCollectorServer.getInstance());
        incrStatusServer.start();
        final List<IOnsListenerStatus> result = new ArrayList<>();
        Collection<IOnsListenerStatus> incrChannels = getAllTransferChannel(result);


        ZkUtils.registerAddress2ZK(// "/tis/incr-transfer-group/incr-state-collect"
                this.zkClient, // "/tis/incr-transfer-group/incr-state-collect"
                ZkUtils.ZK_ASSEMBLE_LOG_COLLECT_PATH, exportPort);
        IncrStatusUmbilicalProtocolImpl.getInstance().startLogging();
        return incrChannels;
    }

    public List<IOnsListenerStatus> getAllTransferChannel(final List<IOnsListenerStatus> result) {
        return result;
    }

    /**
     * 由servlet接收到命令之后触发
     *
     * @param execContext
     * @throws Exception
     */
    @SuppressWarnings("all")
    public ExecuteResult startWork(DefaultChainContext chainContext) throws Exception {
        chainContext.rebindLoggingMDCParams();
        ActionInvocation invoke = null;
        ExecutePhaseRange range = chainContext.getExecutePhaseRange();
        logger.info("start component:" + range.getStart() + ",end component:" + range.getEnd());
        Objects.requireNonNull(this.zkClient, "zkClient can not be null");
        chainContext.setZkClient(this.zkClient);
        invoke = AbstractActionInvocation.createExecChain(chainContext);
        ExecuteResult execResult = invoke.invoke();
        if (!execResult.isSuccess()) {
            logger.warn(execResult.getMessage());
        }
        return execResult;
    }


    @Override
    public void init(DaemonContext context) throws DaemonInitException, Exception {
    }

    @Override
    public void start() throws Exception {
        afterPropertiesSet();
        logger.info("index Swap Task ready");
    }

    public static void main(String[] arg) throws Exception {
        IndexSwapTaskflowLauncher launcher = new IndexSwapTaskflowLauncher();
        launcher.start();
        synchronized (launcher) {
            launcher.wait();
        }
    }

    @Override
    public void stop() throws Exception {
    }

    @Override
    public void destroy() {
    }

    /**
     * @param taskid
     * @return
     * @throws Exception
     */
    public static PhaseStatusCollection loadPhaseStatusFromLocal(int taskid) {
        PhaseStatusCollection result = null;
        FullbuildPhase[] phases = FullbuildPhase.values();
        try {
            File localFile = null;
            BasicPhaseStatus phaseStatus;
            for (FullbuildPhase phase : phases) {
                localFile = BasicPhaseStatus.getFullBuildPhaseLocalFile(taskid, phase);
                if (!localFile.exists()) {
                    return result;
                }
                if (result == null) {
                    result = new PhaseStatusCollection(taskid, ExecutePhaseRange.fullRange());
                }
                IFlush2Local flush2Local = IFlush2LocalFactory.createNew(IndexSwapTaskflowLauncher.class.getClassLoader(), localFile)
                        .orElseThrow(() -> new IllegalStateException("flush2Local must be present"));
                phaseStatus = flush2Local.loadPhase(); // BasicPhaseStatus.statusWriter.loadPhase(localFile);
                switch (phase) {
                    case FullDump:
                        result.setDumpPhase((DumpPhaseStatus) phaseStatus);
                        break;
                    case JOIN:
                        result.setJoinPhase((JoinPhaseStatus) phaseStatus);
                        break;
                    case BUILD:
                        result.setBuildPhase((BuildPhaseStatus) phaseStatus);
                        break;
                    case IndexBackFlow:
                        result.setIndexBackFlowPhaseStatus((IndexBackFlowPhaseStatus) phaseStatus);
                }
            }
        } catch (Exception e) {
            throw new RuntimeException("taskid:" + taskid, e);
        }
        return result;
    }
}
